	include	tios.h
	include	flib.h
	include	hexlib.h

	include genlib.h
	include macros.h

	xdef	_main
	xdef	_comment
	xdef	_ti92plus
	xdef	_ti89

_main:
	jsr	flib::zap_screen	; Clear the screen

	genlib::PUSH_DSCREEN	d0	; Allocate a DSCREEN on the stack (7688 bytes)
	move.l	d0,Screen		; Save its adress
					; you can't allocate more than one screen

	jsr	genlib::init_dscreen	; Allocat a new DScreen for double buffer
	tst.w	d0
	beq	fail			; Memory ?
	move.w	d0,MyHd			; No, save the handle
	move.l	a0,Screen2		; and the adress

	jsr	genlib::init		; Init the grey-mode (or gray-mode ?)

	move.l	Screen(PC),d0
	jsr	genlib::set_dscreen_int		; Set the interrupt to this Dscreen
	move.l	Screen2(PC),d0
	jsr	genlib::set_dscreen_function	; Set the functions to this DScreen

	lea	Plane_Table(PC),a3	; Plane 2 table
	sub.l	a5,a5			; No indirect table
	lea	Plane_Tile(Pc),a4	; Tiles adress
	moveq	#5,d3			; Plane is 32=2^5 bytes long
	jsr	genlib::init_plane	; init the plane
	tst.w	d0			; Test : memory ?
	beq	\Error2			; Yes
	move.l	a0,plane2		; Save the adress of the plane

	move.l	plane2(pc),a0		; Select the plane
	move.w	#10,PLANE_XS(a0)		; Set the Scrolling coordinates
	move.w	#20,PLANE_YS(a0)

	move.w	#1,dir3		; Direction x of plane 2
	move.w	#2,dir4		; Direction y of plane 2

	clr.w	temps		; Clear temps of double-buffer

; Generate a dhz buffer
	lea	Dhz_table,a2
	moveq	#(-3),d0
	bsr	copy_16		; 16
	addq.b	#1,d0
	bsr	copy_16		; 32
	addq.b	#1,d0
	bsr	copy_16		; 48
	addq.b	#1,d0
	bsr	copy_16		; 64
	subq.b	#1,d0
	bsr	copy_16		; 80
	subq.b	#1,d0
	bsr	copy_16		; 96
	subq.b	#1,d0
	bsr	copy_16		; 112
	subq.b	#1,d0
	bsr	copy_16		; 128

; Generate a cleared hdz buffer.
	lea	Hdz_table,a2
	moveq	#SCREEN_Y/2-1,d0
\looph:
	clr.l	(a2)+
	dbf.s	d0,\looph

\Continue:
; change the hdz buffer
;	lea	Hdz_table,a2
;	move.w	(a2),d2
;	moveq	#(SCREEN_Y-2),d1
;\loop89:
;	move.w	2(a2),(a2)+
;	dbf	d1,\loop89
;	move.w	d2,(a2)

; Change the dhz buffer : rotation
	lea	Dhz_table,a2
	move.b	(a2),d3;
	moveq	#(SCREEN_Y-2),d1
\loop2:
	move.b	1(a2),(a2)+
	dbf	d1,\loop2
	move.b	d3,(a2)

; Change the coordinates of the scrolling of plane 2
	move.l	plane2(pc),a0
	move.w	PLANE_XS(a0),d0
	add.w	dir3(pc),d0
	cmp.w	#32*16-240,d0
	bls	\ok3
	neg.w	dir3
	bra.s	\pas_ok3
\ok3:
	move.w	d0,PLANE_XS(a0)
\pas_ok3:
	move.w	PLANE_YS(a0),d0
	add.w	dir4(pc),d0
	cmp.w	#32*16-128,d0	
	bls	\ok4
	neg.w	dir4
	bra.s	\pas_ok4
\ok4:
	move.w	d0,PLANE_YS(a0)
\pas_ok4:

; Put the plane 2 as background
	move.l	plane2(pc),a0
	jsr	genlib::update_vscreen_16
	lea	Dhz_table,a4
	lea	Hdz_table,a5
	jsr	genlib::put_dhz_plane

; Do the double-buffer
	bsr	double_buffer

; Wait next 30Hz
\wait:	cmp.l	#2,genlib::timer
	bls.s	\wait		
	subq.l	#3,genlib::timer

; A key ?
	jsr	genlib::keypressed	; A key ?
	tst.w	d0
	beq	\Continue		; no key

	jsr	genlib::quit		; Disable grey-mode

	move.l	plane2(pc),a0
	jsr	genlib::free_plane	; Free the plane 2
\Error2
	move.w	MyHd(PC),-(a7)
	jsr	tios::HeapFree		; Free the DScreen
	addq.l	#2,a7
fail:
	genlib::POP_DSCREEN		; Pop DScreen
	rts				; Quit

double_buffer:
	move.w	temps(pc),d1		; Read temps
	move.l	Screen(pc,d1.w),d0	; Read a screen
	jsr	genlib::set_dscreen_int	; Set the interrupt to this Dscreen
	eor.w	#4,d1			; Next screen
	move.l	Screen(PC,d1.w),d0	; Read the other screen
	jsr	genlib::set_dscreen_function	; Set the functions to this DScreen
	move.w	d1,temps		; Save temps
	rts

copy_16:
	moveq	#15,d2
\lopp:
	move.b	d0,(a2)+
	dbf	d2,\lopp
	rts

MyHd	dc.w	0		; Handle of double-buffer
Screen	dc.l	0		; Screen 1
Screen2	dc.l	0		; Screen 2
temps	dc.w	0		; Temps
plane	dc.l	0		; Plane 1
plane2	dc.l	0		; Plane 2
dir	dc.w	0		; Dir x1
dir2	dc.w	0		; Dir y1
dir3	dc.w	0		; Dir x2
dir4	dc.w	0		; Dir y2

Plane_Tile:
	incbin "decor.bin"

Plane_Table:
	incbin "pl.lvl"

_comment
	dc.b	"Example 5",0
	BSS
Dhz_table:
	dcb.b	SCREEN_Y,0
Hdz_table:
	dcb.w	SCREEN_Y,0

	end
